@@ -9,11 +9,11 @@
 #include <string>
 #include <unordered_map>
 #include "cifdoc.hpp"
-#include "fail.hpp"   // for fail
-#include "numb.hpp"   // for as_number
-#include "atox.hpp"   // for string_to_int
+#include "fail.hpp"    // for fail
+#include "numb.hpp"    // for as_number
+#include "atox.hpp"    // for string_to_int
 #include "model.hpp"
-#include "entstr.hpp" // for entity_type_from_string, polymer_type_from_string
+#include "enumstr.hpp" // for entity_type_from_string, polymer_type_from_string
 #include "mmcif_impl.hpp" // for set_cell_from_mmcif
 
 namespace gemmi {
@@ -51,7 +51,7 @@ get_anisotropic_u(cif::Block& block) {
 }
 
 inline
-std::vector<std::string> transform_tags(std::string mstr, std::string vstr) {
+std::vector<std::string> transform_tags(const std::string& mstr, const std::string& vstr) {
   return {mstr + "[1][1]", mstr + "[1][2]", mstr + "[1][3]", vstr + "[1]",
           mstr + "[2][1]", mstr + "[2][2]", mstr + "[2][3]", vstr + "[2]",
           mstr + "[3][1]", mstr + "[3][2]", mstr + "[3][3]", vstr + "[3]"};
@@ -67,27 +67,30 @@ inline Transform get_transform_matrix(const cif::Table::Row& r) {
   return t;
 }
 
-inline ResidueId make_resid(const std::string& name,
-                            const std::string& seqid,
-                            const std::string* icode) {
-  ResidueId rid;
-  rid.name = name;
+inline SeqId make_seqid(const std::string& seqid, const std::string* icode) {
+  SeqId ret;
   if (icode)
     // the insertion code happens to be always a single letter
-    rid.seqid.icode = cif::as_char(*icode, ' ');
+    ret.icode = cif::as_char(*icode, ' ');
   if (!seqid.empty()) {
     // old mmCIF files have auth_seq_id as number + icode (e.g. 15A)
     if (seqid.back() >= 'A') {
-      if (rid.seqid.icode == ' ')
-        rid.seqid.icode = seqid.back();
-      else if (rid.seqid.icode != seqid.back())
+      if (ret.icode == ' ')
+        ret.icode = seqid.back();
+      else if (ret.icode != seqid.back())
         fail("Inconsistent insertion code in " + seqid);
-      rid.seqid.num = cif::as_int(seqid.substr(0, seqid.size() - 1));
+      ret.num = cif::as_int(seqid.substr(0, seqid.size() - 1));
     } else {
-      rid.seqid.num = cif::as_int(seqid, Residue::OptionalNum::None);
+      ret.num = cif::as_int(seqid, Residue::OptionalNum::None);
     }
   }
-  return rid;
+  return ret;
+}
+
+inline ResidueId make_resid(const std::string& name,
+                            const std::string& seqid,
+                            const std::string* icode) {
+  return ResidueId{make_seqid(seqid, icode), {}, name};
 }
 
 inline std::vector<Helix> read_helices(cif::Block& block) {
@@ -211,12 +214,7 @@ inline void read_connectivity(cif::Block& block, Structure& st) {
     Connection c;
     c.name = row.str(kId);
     copy_string(row, kLinkId, c.link_id);
-    std::string type = row.str(kConnTypeId);
-    for (int i = 0; i != Connection::Unknown; ++i)
-      if (get_mmcif_connection_type_id(Connection::Type(i)) == type) {
-        c.type = Connection::Type(i);
-        break;
-      }
+    c.type = connection_type_from_string(row.str(kConnTypeId));
     if (row.has2(kSym1) && row.has2(kSym2)) {
       c.asu = (row.str(kSym1) == row.str(kSym2) ? Asu::Same : Asu::Different);
     }
@@ -278,7 +276,7 @@ inline std::vector<Assembly> read_assemblies(cif::Block& block) {
                                    {"biol_id", "type", "value"});
   cif::Table gen_tab = block.find("_pdbx_struct_assembly_gen.",
                           {"assembly_id", "oper_expression", "asym_id_list"});
-  std::vector<Assembly::Oper> oper_list;
+  std::vector<Assembly::Operator> oper_list;
   std::vector<std::string> oper_list_tags = transform_tags("matrix", "vector");
   oper_list_tags.emplace_back("id");  // 12
   oper_list_tags.emplace_back("type");  // 13
@@ -333,8 +331,8 @@ inline std::vector<Assembly> read_assemblies(cif::Block& block) {
         Assembly::Gen& gen = a.generators.back();
         split_str_into(row_g.str(2), ',', gen.subchains);
         for (const std::string& name : parse_operation_expr(row_g.str(1)))
-          if (const Assembly::Oper* oper = impl::find_or_null(oper_list, name))
-            gen.opers.push_back(*oper);
+          if (const Assembly::Operator* oper = impl::find_or_null(oper_list, name))
+            gen.operators.push_back(*oper);
       }
   }
   return assemblies;
@@ -370,24 +368,22 @@ DiffractionInfo* find_diffrn(Metadata& meta, const std::string& diffrn_id) {
 }
 
 inline Structure make_structure_from_block(const cif::Block& block_) {
-  using cif::as_number;
-  using cif::as_string;
   // find() and Table don't have const variants, but we don't change anything.
   cif::Block& block = const_cast<cif::Block&>(block_);
   Structure st;
   st.input_format = CoorFormat::Mmcif;
   st.name = block.name;
   set_cell_from_mmcif(block, st.cell);
-  st.spacegroup_hm = as_string(impl::find_spacegroup_hm_value(block));
+  st.spacegroup_hm = cif::as_string(impl::find_spacegroup_hm_value(block));
 
-  auto add_info = [&](std::string tag) {
+  auto add_info = [&](const std::string& tag) {
     bool first = true;
     for (const std::string& v : block.find_values(tag))
       if (!cif::is_null(v)) {
         if (first)
-          st.info[tag] = as_string(v);
+          st.info[tag] = cif::as_string(v);
         else
-          st.info[tag] += "; " + as_string(v);
+          st.info[tag] += "; " + cif::as_string(v);
         first = false;
       }
   };
@@ -533,7 +529,10 @@ inline Structure make_structure_from_block(const cif::Block& block_) {
   for (auto op : ncs_oper) {
     bool given = op.has(13) && op.str(13) == "given";
     Transform tr = get_transform_matrix(op);
-    if (!tr.is_identity())
+    if (tr.is_identity())
+      // ignore identity, but store its id so we can write it back to mmCIF
+      st.info["_struct_ncs_oper.id"] = op.str(12);
+    else
       st.ncs.push_back({op.str(12), given, tr});
   }
 
@@ -560,14 +559,15 @@ inline Structure make_structure_from_block(const cif::Block& block_) {
   // atom list
   enum { kId=0, kGroupPdb, kSymbol, kLabelAtomId, kAltId, kLabelCompId,
          kLabelAsymId, kLabelSeqId, kInsCode, kX, kY, kZ, kOcc, kBiso, kCharge,
-         kAuthSeqId, kAuthCompId, kAuthAsymId, kAuthAtomId, kModelNum };
+         kAuthSeqId, kAuthCompId, kAuthAsymId, kAuthAtomId, kModelNum,
+         kCalcFlag, kTlsGroupId };
   cif::Table atom_table = block.find("_atom_site.",
                                      {"id",
                                       "?group_PDB",
                                       "type_symbol",
-                                      "label_atom_id",
+                                      "?label_atom_id",
                                       "label_alt_id",
-                                      "label_comp_id",
+                                      "?label_comp_id",
                                       "label_asym_id",
                                       "?label_seq_id",
                                       "?pdbx_PDB_ins_code",
@@ -581,74 +581,95 @@ inline Structure make_structure_from_block(const cif::Block& block_) {
                                       "?auth_comp_id",
                                       "?auth_asym_id",
                                       "?auth_atom_id",
-                                      "?pdbx_PDB_model_num"});
-  const int kCompId = atom_table.has_column(kAuthCompId) ? kAuthCompId
-                                                         : kLabelCompId;
-  const int kAsymId = atom_table.has_column(kAuthAsymId) ? kAuthAsymId
-                                                         : kLabelAsymId;
-  const int kAtomId = atom_table.has_column(kAuthAtomId) ? kAuthAtomId
-                                                         : kLabelAtomId;
-  Model *model = nullptr;
-  Chain *chain = nullptr;
-  Residue *resi = nullptr;
+                                      "?pdbx_PDB_model_num",
+                                      "?calc_flag",
+                                      "?pdbx_tls_group_id",
+                                     });
   if (atom_table.length() != 0) {
+    const int kAsymId = atom_table.first_of(kAuthAsymId, kLabelAsymId);
+    // we use only one comp (residue) and one atom name
+    const int kCompId = atom_table.first_of(kAuthCompId, kLabelCompId);
+    const int kAtomId = atom_table.first_of(kAuthAtomId, kLabelAtomId);
+    if (!atom_table.has_column(kCompId))
+      fail("Neither _atom_site.label_comp_id nor auth_comp_id found");
+    if (!atom_table.has_column(kAtomId))
+      fail("Neither _atom_site.label_atom_id nor auth_atom_id found");
+
+    Model *model = nullptr;
+    Chain *chain = nullptr;
+    Residue *resi = nullptr;
     if (atom_table.has_column(kModelNum))
       model = &st.find_or_add_model(atom_table[0].str(kModelNum));
     else
       model = &st.find_or_add_model("1");
-  }
-  for (auto row : atom_table) {
-    if (row.has(kModelNum) && row[kModelNum] != model->name) {
-      model = &st.find_or_add_model(row.str(kModelNum));
-      chain = nullptr;
-    }
-    if (!chain || as_string(row[kAsymId]) != chain->name) {
-      model->chains.emplace_back(as_string(row[kAsymId]));
-      chain = &model->chains.back();
-      resi = nullptr;
-    }
-    ResidueId rid = make_resid(as_string(row[kCompId]),
-                               as_string(row[kAuthSeqId]),
-                               row.has(kInsCode) ? &row[kInsCode] : nullptr);
-    if (!resi || !resi->matches(rid)) {
-      resi = chain->find_or_add_residue(rid);
-      if (resi->atoms.empty()) {
-        if (row.has2(kLabelSeqId))
-          resi->label_seq = cif::as_int(row[kLabelSeqId]);
-        resi->subchain = row.str(kLabelAsymId);
-        // don't check if group_PDB is consistent, it's not that important
-        if (row.has2(kGroupPdb))
-          for (int i = 0; i < 2; ++i) { // first character could be " or '
-            const char c = alpha_up(row[kGroupPdb][i]);
-            if (c == 'A' || c == 'H' || c == '\0')
-              resi->het_flag = c;
-          }
+    for (auto row : atom_table) {
+      if (row.has(kModelNum) && row[kModelNum] != model->name) {
+        model = &st.find_or_add_model(row.str(kModelNum));
+        chain = nullptr;
       }
-    } else if (resi->seqid != rid.seqid) {
-      fail("Inconsistent sequence ID: " + resi->str() + " / " + rid.str());
-    }
-    Atom atom;
-    atom.name = as_string(row[kAtomId]);
-    // altloc is always a single letter (not guaranteed by the mmCIF spec)
-    atom.altloc = cif::as_char(row[kAltId], '\0');
-    atom.charge = row.has2(kCharge) ? cif::as_int(row[kCharge]) : 0;
-    atom.element = gemmi::Element(as_string(row[kSymbol]));
-    // According to the PDBx/mmCIF spec _atom_site.id can be a string,
-    // but in all the files it is a serial number; its value is not essential,
-    // so we just ignore non-integer ids.
-    atom.serial = string_to_int(row[kId], false);
-    atom.pos.x = cif::as_number(row[kX]);
-    atom.pos.y = cif::as_number(row[kY]);
-    atom.pos.z = cif::as_number(row[kZ]);
-    atom.occ = (float) cif::as_number(row[kOcc], 1.0);
-    atom.b_iso = (float) cif::as_number(row[kBiso], 50.0);
+      if (!chain || cif::as_string(row[kAsymId]) != chain->name) {
+        model->chains.emplace_back(cif::as_string(row[kAsymId]));
+        chain = &model->chains.back();
+        resi = nullptr;
+      }
+      ResidueId rid = make_resid(cif::as_string(row[kCompId]),
+                                 cif::as_string(row[kAuthSeqId]),
+                                 row.has(kInsCode) ? &row[kInsCode] : nullptr);
+      if (!resi || !resi->matches(rid)) {
+        resi = chain->find_or_add_residue(rid);
+        if (resi->atoms.empty()) {
+          if (row.has2(kLabelSeqId))
+            resi->label_seq = cif::as_int(row[kLabelSeqId]);
+          resi->subchain = row.str(kLabelAsymId);
+          // don't check if group_PDB is consistent, it's not that important
+          if (row.has2(kGroupPdb))
+            for (int i = 0; i < 2; ++i) { // first character could be " or '
+              const char c = alpha_up(row[kGroupPdb][i]);
+              if (c == 'A' || c == 'H' || c == '\0')
+                resi->het_flag = c;
+            }
+        }
+      } else if (resi->seqid != rid.seqid) {
+        fail("Inconsistent sequence ID: " + resi->str() + " / " + rid.str());
+      }
+      Atom atom;
+      atom.name = cif::as_string(row[kAtomId]);
+      // altloc is always a single letter (not guaranteed by the mmCIF spec)
+      atom.altloc = cif::as_char(row[kAltId], '\0');
+      atom.charge = row.has2(kCharge) ? cif::as_int(row[kCharge]) : 0;
+      atom.element = gemmi::Element(cif::as_string(row[kSymbol]));
+      // According to the PDBx/mmCIF spec _atom_site.id can be a string,
+      // but in all the files it is a serial number; its value is not essential,
+      // so we just ignore non-integer ids.
+      atom.serial = string_to_int(row[kId], false);
+      if (row.has2(kCalcFlag)) {
+        const std::string& cf = row[kCalcFlag];
+        if (cf[0] == 'c')
+          atom.calc_flag = CalcFlag::Calculated;
+        if (cf[0] == 'd')
+          atom.calc_flag = cf[1] == 'u' ? CalcFlag::Dummy
+                                        : CalcFlag::Determined;
+      }
+      if (row.has2(kTlsGroupId)) {
+        const char* str = row[kTlsGroupId].c_str();
+        const char* endptr;
+        int tls_id = no_sign_atoi(str, &endptr);
+        if (endptr != str)
+          atom.tls_group_id = (short) tls_id;
+      }
+      atom.pos.x = cif::as_number(row[kX]);
+      atom.pos.y = cif::as_number(row[kY]);
+      atom.pos.z = cif::as_number(row[kZ]);
+      atom.occ = (float) cif::as_number(row[kOcc], 1.0);
+      atom.b_iso = (float) cif::as_number(row[kBiso], 50.0);
 
-    if (!aniso_map.empty()) {
-      auto ani = aniso_map.find(row[kId]);
-      if (ani != aniso_map.end())
-        atom.aniso = ani->second;
+      if (!aniso_map.empty()) {
+        auto ani = aniso_map.find(row[kId]);
+        if (ani != aniso_map.end())
+          atom.aniso = ani->second;
+      }
+      resi->atoms.emplace_back(atom);
     }
-    resi->atoms.emplace_back(atom);
   }
 
   cif::Table polymer_types = block.find("_entity_poly.", {"entity_id", "type"});
@@ -676,12 +697,27 @@ inline Structure make_structure_from_block(const cif::Block& block_) {
         ent->full_sequence[pos] += "," + row.str(2);
     }
 
+  cif::Table struct_ref = block.find("_struct_ref.",
+      {"id", "entity_id", "db_name", "db_code",
+       "?pdbx_db_accession", "?pdbx_db_isoform"});
   cif::Table struct_ref_seq = block.find("_struct_ref_seq.",
-                                {"ref_id", "seq_align_beg", "seq_align_end",
-                                 "db_align_beg", "db_align_end"});
-  for (auto row : block.find("_struct_ref.",
-                             {"id", "entity_id", "db_name", "db_code",
-                              "?pdbx_db_accession", "?pdbx_db_isoform"}))
+      {"ref_id", "seq_align_beg", "seq_align_end",                   // 0-2
+       "db_align_beg", "db_align_end",                               // 3-4
+       "?pdbx_auth_seq_align_beg", "?pdbx_seq_align_beg_ins_code",   // 5-6
+       "?pdbx_auth_seq_align_end", "?pdbx_seq_align_end_ins_code"}); // 7-8
+  // DbRef doesn't correspond 1:1 to the mmCIF tables; we need to remove
+  // duplicates from _struct_ref_seq to make it work.
+  std::vector<std::string> seen;
+  for (cif::Table::Row seq : struct_ref_seq) {
+    std::string str = seq[0];
+    for (int i = 1; i < 5; ++i) {
+      str += '\t';
+      str += seq[i];
+    }
+    if (in_vector(str, seen))
+      continue;
+    seen.push_back(str);
+    cif::Table::Row row = struct_ref.find_row(seq.str(0));
     if (Entity* ent = st.get_entity(row.str(1))) {
       ent->dbrefs.emplace_back();
       Entity::DbRef& dbref = ent->dbrefs.back();
@@ -691,15 +727,17 @@ inline Structure make_structure_from_block(const cif::Block& block_) {
         dbref.accession_code = row.str(4);
       if (row.has(5))
         dbref.isoform = row.str(5);
-      try { // find_row() throws if row is not found
-        cif::Table::Row seq = struct_ref_seq.find_row(row.str(0));
-        constexpr int None = SeqId::OptionalNum::None;
-        dbref.label_seq_begin = cif::as_int(seq[1], None);
-        dbref.label_seq_end = cif::as_int(seq[2], None);
-        dbref.db_begin.num = cif::as_int(seq[3], None);
-        dbref.db_end.num = cif::as_int(seq[4], None);
-      } catch (const std::runtime_error&) {}
+      constexpr int None = SeqId::OptionalNum::None;
+      dbref.label_seq_begin = cif::as_int(seq[1], None);
+      dbref.label_seq_end = cif::as_int(seq[2], None);
+      dbref.db_begin.num = cif::as_int(seq[3], None);
+      dbref.db_end.num = cif::as_int(seq[4], None);
+      if (seq.has(5))
+        dbref.seq_begin = make_seqid(seq.str(5), seq.ptr_at(6));
+      if (seq.has(7))
+        dbref.seq_end = make_seqid(seq.str(7), seq.ptr_at(8));
     }
+  }
 
   for (auto row : block.find("_struct_asym.", {"id", "entity_id"}))
     if (Entity* ent = st.get_entity(row.str(1)))
@@ -738,6 +776,15 @@ inline Structure make_structure_from_block(const cif::Block& block) {
   return impl::make_structure_from_block(block);
 }
 
+inline Structure make_structure(const cif::Document& doc) {
+  // mmCIF files for deposition may have more than one block:
+  // coordinates in the first block and restraints in the others.
+  for (size_t i = 1; i < doc.blocks.size(); ++i)
+    if (doc.blocks[i].has_tag("_atom_site.id"))
+      fail("2+ blocks are ok if only the first one has coordinates;\n"
+           "_atom_site in block #" + std::to_string(i+1) + ": " + doc.source);
+  return make_structure_from_block(doc.blocks.at(0));
+}
 
 } // namespace gemmi
 #endif
