@@ -55,6 +55,13 @@ struct Topo {
       return calculate_chiral_volume(atoms[0]->pos, atoms[1]->pos,
                                      atoms[2]->pos, atoms[3]->pos);
     }
+    double calculate_z(double ideal_abs_vol, double esd) const {
+      double calc = calculate();
+      if (restr->sign == ChiralityType::Negative ||
+          (restr->sign == ChiralityType::Both && calc < 0))
+        ideal_abs_vol *= -1;
+      return std::abs(calc - ideal_abs_vol) / esd;
+    }
     bool check() const { return !restr->is_wrong(calculate()); }
   };
   struct Plane {
@@ -67,7 +74,7 @@ struct Topo {
 
   enum class Provenance { None, PrevLink, Monomer, NextLink, ExtraLink };
   enum class RKind { Bond, Angle, Torsion, Chirality, Plane };
-  struct Force {
+  struct Rule {
     Provenance provenance;
     RKind rkind;
     size_t index; // index in the respective vector (bonds, ...) in Topo
@@ -77,14 +84,15 @@ struct Topo {
     Residue* res;
     struct Prev {
       std::string link;
-      int idx; // relative to current index, 0 means n/a
+      int idx; // relative to current index
       const ResInfo* get(const ResInfo* t) const { return t + idx; }
       ResInfo* get(ResInfo* t) const { return t + idx; }
     };
+    // in case of microheterogeneity we may have 2+ previous residues
     std::vector<Prev> prev;
     std::vector<std::string> mods;
     ChemComp chemcomp;
-    std::vector<Force> forces;
+    std::vector<Rule> rules;
 
     ResInfo(Residue* r) : res(r) {}
     void add_mod(const std::string& m) {
@@ -120,7 +128,7 @@ struct Topo {
     char alt1 = '\0';
     char alt2 = '\0';
     std::string link_id;
-    std::vector<Force> forces;
+    std::vector<Rule> rules;
   };
 
   template<typename T>
@@ -174,9 +182,22 @@ struct Topo {
     return nullptr;
   }
 
-  std::vector<Force> apply_restraints(const Restraints& rt,
-                                      Residue& res, Residue* res2,
-                                      char altloc='*') {
+  double ideal_chiral_abs_volume(const Chirality &ch) const {
+    const Restraints::Bond* bond_c1 = take_bond(ch.atoms[0], ch.atoms[1]);
+    const Restraints::Bond* bond_c2 = take_bond(ch.atoms[0], ch.atoms[2]);
+    const Restraints::Bond* bond_c3 = take_bond(ch.atoms[0], ch.atoms[3]);
+    const Restraints::Angle* angle_1c2 = take_angle(ch.atoms[1], ch.atoms[0], ch.atoms[2]);
+    const Restraints::Angle* angle_2c3 = take_angle(ch.atoms[2], ch.atoms[0], ch.atoms[3]);
+    const Restraints::Angle* angle_3c1 = take_angle(ch.atoms[3], ch.atoms[0], ch.atoms[1]);
+    if (bond_c1 && bond_c2 && bond_c3 && angle_1c2 && angle_2c3 && angle_3c1)
+      return chiral_abs_volume(bond_c1->value, bond_c2->value, bond_c3->value,
+                               angle_1c2->value, angle_2c3->value, angle_3c1->value);
+    return std::numeric_limits<double>::quiet_NaN();
+  }
+
+  std::vector<Rule> apply_restraints(const Restraints& rt,
+                                     Residue& res, Residue* res2,
+                                     char altloc='*') {
     std::string altlocs;
     if (altloc == '*') {
       // find all distinct altlocs
@@ -187,13 +208,13 @@ struct Topo {
     if (altlocs.empty())
       altlocs += altloc;
 
-    std::vector<Force> forces;
+    std::vector<Rule> rules;
     Provenance pro = Provenance::None;
     for (const Restraints::Bond& bond : rt.bonds)
       for (char alt : altlocs)
         if (Atom* at1 = bond.id1.get_from(res, res2, alt))
           if (Atom* at2 = bond.id2.get_from(res, res2, alt)) {
-            forces.push_back({pro, RKind::Bond, bonds.size()});
+            rules.push_back({pro, RKind::Bond, bonds.size()});
             bonds.push_back({&bond, {{at1, at2}}});
             if (!at1->altloc && !at2->altloc)
               break;
@@ -203,7 +224,7 @@ struct Topo {
         if (Atom* at1 = angle.id1.get_from(res, res2, alt))
           if (Atom* at2 = angle.id2.get_from(res, res2, alt))
             if (Atom* at3 = angle.id3.get_from(res, res2, alt)) {
-              forces.push_back({pro, RKind::Angle, angles.size()});
+              rules.push_back({pro, RKind::Angle, angles.size()});
               angles.push_back({&angle, {{at1, at2, at3}}});
               if (!at1->altloc && !at2->altloc && !at3->altloc)
                 break;
@@ -214,7 +235,7 @@ struct Topo {
           if (Atom* at2 = tor.id2.get_from(res, res2, alt))
             if (Atom* at3 = tor.id3.get_from(res, res2, alt))
               if (Atom* at4 = tor.id4.get_from(res, res2, alt)) {
-                forces.push_back({pro, RKind::Torsion, torsions.size()});
+                rules.push_back({pro, RKind::Torsion, torsions.size()});
                 torsions.push_back({&tor, {{at1, at2, at3, at4}}});
                 if (!at1->altloc && !at2->altloc &&
                     !at3->altloc && !at4->altloc)
@@ -226,7 +247,7 @@ struct Topo {
           if (Atom* at2 = chir.id1.get_from(res, res2, alt))
             if (Atom* at3 = chir.id2.get_from(res, res2, alt))
               if (Atom* at4 = chir.id3.get_from(res, res2, alt)) {
-                forces.push_back({pro, RKind::Chirality, chirs.size()});
+                rules.push_back({pro, RKind::Chirality, chirs.size()});
                 chirs.push_back({&chir, {{at1, at2, at3, at4}}});
                 if (!at1->altloc && !at2->altloc &&
                     !at3->altloc && !at4->altloc)
@@ -239,30 +260,30 @@ struct Topo {
           if (Atom* atom = id.get_from(res, res2, alt))
             atoms.push_back(atom);
         if (atoms.size() >= 4) {
-          forces.push_back({pro, RKind::Plane, planes.size()});
+          rules.push_back({pro, RKind::Plane, planes.size()});
           planes.push_back({&plane, atoms});
         }
         if (std::all_of(atoms.begin(), atoms.end(),
                         [](Atom* a) { return !a->altloc; }))
           break;
       }
-    return forces;
+    return rules;
   }
 
   void apply_internal_restraints_to_residue(ResInfo& ri) {
-    auto forces = apply_restraints(ri.chemcomp.rt, *ri.res, nullptr);
-    for (const auto& f : forces)
-      ri.forces.push_back({Provenance::Monomer, f.rkind, f.index});
+    auto rules = apply_restraints(ri.chemcomp.rt, *ri.res, nullptr);
+    for (const auto& rule : rules)
+      ri.rules.push_back({Provenance::Monomer, rule.rkind, rule.index});
   }
 
   void apply_restraints_to_residue(ResInfo& ri, const MonLib& monlib) {
     for (ResInfo::Prev& prev : ri.prev) {
       if (const ChemLink* link = monlib.find_link(prev.link)) {
         ResInfo* prev_ri = prev.get(&ri);
-        auto forces = apply_restraints(link->rt, *prev_ri->res, ri.res);
-        for (const auto& f : forces) {
-          ri.forces.push_back({Provenance::PrevLink, f.rkind, f.index});
-          prev_ri->forces.push_back({Provenance::NextLink, f.rkind, f.index});
+        auto rules = apply_restraints(link->rt, *prev_ri->res, ri.res);
+        for (const auto& rule : rules) {
+          ri.rules.push_back({Provenance::PrevLink, rule.rkind, rule.index});
+          prev_ri->rules.push_back({Provenance::NextLink, rule.rkind, rule.index});
         }
       }
     }
@@ -282,14 +303,14 @@ struct Topo {
     char alt = link.alt1 ? link.alt1 : link.alt2;
     ResInfo* ri1 = find_resinfo(link.res1);
     ResInfo* ri2 = find_resinfo(link.res2);
-    auto forces = apply_restraints(cl->rt, *link.res1, link.res2, alt);
-    for (Force& f : forces) {
-      f.provenance = Provenance::ExtraLink;
-      link.forces.push_back(f);
+    auto rules = apply_restraints(cl->rt, *link.res1, link.res2, alt);
+    for (Rule& rule : rules) {
+      rule.provenance = Provenance::ExtraLink;
+      link.rules.push_back(rule);
       if (ri1)
-        ri1->forces.push_back(f);
+        ri1->rules.push_back(rule);
       if (ri2)
-        ri2->forces.push_back(f);
+        ri2->rules.push_back(rule);
     }
   }
 
@@ -373,7 +394,7 @@ inline void Topo::ChainInfo::add_refmac_builtin_modifications() {
         front.mods.emplace_back("NH3");
       back.mods.emplace_back(back.res->find_atom("OXT", '*') ? "COO" : "TERMINUS");
     } else if (is_polynucleotide(polymer_type)) {
-      front.mods.emplace_back("5*END");
+      front.mods.emplace_back(front.res->find_atom("P", '*') ? "p5*END" : "5*END");
       back.mods.emplace_back("TERMINUS");
     }
   }
@@ -422,9 +443,20 @@ inline void Topo::initialize_refmac_topology(const Structure& st, Model& model0,
       continue;
     extra.alt1 = conn.partner1.altloc;
     extra.alt2 = conn.partner2.altloc;
-    const ChemLink* match =
-        monlib.match_link(extra.res1->name, conn.partner1.atom_name,
-                          extra.res2->name, conn.partner2.atom_name);
+
+    // first try to find ChemLink by name (and check if it matches)
+    const ChemLink* match = monlib.find_link(conn.link_id);
+    if (match && (
+          match->rt.bonds.empty() ||
+          match->rt.bonds[0].id1.atom != conn.partner1.atom_name ||
+          match->rt.bonds[0].id2.atom != conn.partner2.atom_name ||
+          !monlib.link_side_matches_residue(match->side1, *extra.res1) ||
+          !monlib.link_side_matches_residue(match->side2, *extra.res2)))
+      match = nullptr;
+    // if ChemLink was not found, use the first matching link (if any)
+    if (!match)
+      match = monlib.match_link(extra.res1->name, conn.partner1.atom_name,
+                                extra.res2->name, conn.partner2.atom_name);
     if (!match) {
       match = monlib.match_link(extra.res2->name, conn.partner2.atom_name,
                                 extra.res1->name, conn.partner1.atom_name);
@@ -433,12 +465,14 @@ inline void Topo::initialize_refmac_topology(const Structure& st, Model& model0,
         std::swap(extra.alt1, extra.alt2);
       }
     }
+
     if (match) {
       extra.link_id = match->id;
       // add modifications from the link
       find_resinfo(extra.res1)->add_mod(match->side1.mod);
       find_resinfo(extra.res2)->add_mod(match->side2.mod);
     } else {
+      // create a new ChemLink and add it to the monomer library
       ChemLink cl;
       cl.side1.comp = extra.res1->name;
       cl.side2.comp = extra.res2->name;
