@@ -53,7 +53,6 @@ void add_mol(py::module& m) {
   py::class_<Model> pyModel(m, "Model");
   py::class_<ResidueSpan> pyResidueSpan(m, "ResidueSpan");
   py::class_<ResidueGroup, ResidueSpan> pyResidueGroup(m, "ResidueGroup");
-  py::class_<CraProxy> pyCraGenerator(m, "CraGenerator");
 
   py::enum_<HowToNameCopiedChain>(m, "HowToNameCopiedChain")
     .value("Short", HowToNameCopiedChain::Short)
@@ -120,6 +119,7 @@ void add_mol(py::module& m) {
     .def("renumber_models", &Structure::renumber_models)
     .def("merge_chain_parts", &Structure::merge_chain_parts,
          py::arg("min_sep")=0)
+    .def("remove_empty_chains", &Structure::remove_empty_chains)
     .def("setup_cell_images", &Structure::setup_cell_images)
     .def("add_entity_types", (void (*)(Structure&, bool)) &add_entity_types,
          py::arg("overwrite")=false)
@@ -134,7 +134,6 @@ void add_mol(py::module& m) {
     .def("remove_waters", remove_waters<Structure>)
     .def("remove_ligands_and_waters",
          (void (*)(Structure&)) &remove_ligands_and_waters)
-    .def("remove_empty_chains", (void (*)(Structure&)) &remove_empty_chains)
     .def("shorten_chain_names", &shorten_chain_names)
     .def("expand_ncs", &expand_ncs, py::arg("how"))
     .def("calculate_box", &calculate_box, py::arg("margin")=0.)
@@ -151,11 +150,18 @@ void add_mol(py::module& m) {
     .def_readonly("chain", &CRA::chain)
     .def_readonly("residue", &CRA::residue)
     .def_readonly("atom", &CRA::atom)
+    .def("atom_matches", [](const CRA& self, const AtomAddress& addr) {
+        return atom_matches(self, addr);
+    })
     .def("__str__", [](const CRA& self) { return atom_str(self); })
     .def("__repr__", [](const CRA& self) {
         return tostr("<gemmi.CRA ", atom_str(self), '>');
     });
 
+  py::class_<CraProxy>(m, "CraGenerator")
+    .def("__iter__", [](CraProxy& self) { return py::make_iterator(self); },
+         py::keep_alive<0, 1>());
+
   pyModel
     .def(py::init<std::string>())
     .def_readwrite("name", &Model::name)
@@ -213,7 +219,7 @@ void add_mol(py::module& m) {
         return calculate_center_of_mass(self).get();
     })
     .def("transform", [](Model& self, const Transform& tr) {
-        for (CRA& cra : self.all())
+        for (CRA cra : self.all())
           transform_atom(*cra.atom, tr);
     }, py::arg("tr"))
     .def("split_chains_by_segments", &split_chains_by_segments)
@@ -243,10 +249,6 @@ void add_mol(py::module& m) {
         return py::make_iterator(self);
     }, py::keep_alive<0, 1>());
 
-  pyCraGenerator
-    .def("__iter__", [](CraProxy& self) { return py::make_iterator(self); },
-         py::keep_alive<0, 1>());
-
   pyChain
     .def(py::init<std::string>())
     .def_readwrite("name", &Chain::name)
@@ -288,6 +290,9 @@ void add_mol(py::module& m) {
     .def("count_atom_sites", &count_atom_sites<Chain>)
     .def("count_occupancies", &count_occupancies<Chain>)
     .def("calculate_mass", &calculate_mass<Chain>)
+    .def("calculate_center_of_mass", [](const Chain& self) {
+          return calculate_center_of_mass(self).get();
+     })
     .def("trim_to_alanine", (void (*)(Chain&)) &trim_to_alanine)
     .def("first_conformer",
          (UniqProxy<Residue> (Chain::*)()) &Chain::first_conformer,
@@ -441,6 +446,7 @@ void add_mol(py::module& m) {
     .def("is_hydrogen", &Atom::is_hydrogen)
     .def("has_altloc", &Atom::has_altloc)
     .def("b_eq", &Atom::b_eq)
+    .def("padded_name", &Atom::padded_name)
     .def("clone", [](const Atom& self) { return new Atom(self); })
     .def("__repr__", [](const Atom& self) {
         std::string r = "<gemmi.Atom " + self.name;
@@ -473,6 +479,5 @@ void add_mol(py::module& m) {
   m.def("make_assembly", [](const Assembly& assembly, const Model& model,
                             HowToNameCopiedChain how) {
         return make_assembly(assembly, model, how, nullptr);
-  })
-  ;
+  });
 }
