@@ -2,6 +2,7 @@
 
 #include "gemmi/unitcell.hpp"
 #include "gemmi/tostr.hpp"  // tostr
+#include "gemmi/gruber.hpp"  // GruberVector
 
 #include <cstdio>  // for snprintf
 #include <array>
@@ -32,6 +33,10 @@ static py::tuple make_parameters_tuple(const UnitCell& u) {
   return py::make_tuple(u.a, u.b, u.c, u.alpha, u.beta, u.gamma);
 }
 
+static py::tuple make_six_tuple(const std::array<double,6>& v) {
+  return py::make_tuple(v[0], v[1], v[2], v[3], v[4], v[5]);
+}
+
 template<typename T> void add_smat33(py::module& m, const char* name) {
   using M = SMat33<T>;
   py::class_<M>(m, name)
@@ -45,7 +50,8 @@ template<typename T> void add_smat33(py::module& m, const char* name) {
     .def_readwrite("u12", &M::u12)
     .def_readwrite("u13", &M::u13)
     .def_readwrite("u23", &M::u23)
-    .def("elements", &M::elements)
+    .def("elements_pdb", &M::elements_pdb)
+    .def("elements_voigt", &M::elements_voigt)
     .def("as_mat33", &M::as_mat33)
     .def("trace", &M::trace)
     .def("nonzero", &M::nonzero)
@@ -92,6 +98,7 @@ void add_unitcell(py::module& m) {
     .def_readwrite("z", &Vec3::z)
     .def("dot", &Vec3::dot)
     .def("cross", &Vec3::cross)
+    .def("length", &Vec3::length)
     .def("approx", &Vec3::approx, py::arg("other"), py::arg("epsilon"))
     .def("tolist", [](const Vec3& self) {
         return std::array<double,3>{{self.x, self.y, self.z}};
@@ -119,7 +126,8 @@ void add_unitcell(py::module& m) {
     .def("__repr__", [](const Vec3& self) {
         return "<gemmi.Vec3(" + triple(self.x, self.y, self.z) + ")>";
     });
-  py::class_<Mat33>(m, "Mat33", py::buffer_protocol())
+  py::class_<Mat33> mat33(m, "Mat33", py::buffer_protocol());
+  mat33
     .def(py::init<>())
     .def(py::init([](std::array<std::array<double,3>,3>& m) {
        Mat33 *mat = new Mat33();
@@ -131,6 +139,8 @@ void add_unitcell(py::module& m) {
                              {3, 3}, // dimensions
                              {sizeof(double)*3, sizeof(double)});  // strides
     })
+    .def("row_copy", &Mat33::row_copy)
+    .def("column_copy", &Mat33::column_copy)
     .def("multiply", (Mat33 (Mat33::*)(const Mat33&) const) &Mat33::multiply)
     .def("multiply", (Vec3 (Mat33::*)(const Vec3&) const) &Mat33::multiply)
     .def("left_multiply", &Mat33::left_multiply)
@@ -151,6 +161,7 @@ void add_unitcell(py::module& m) {
                "             [" + triple(a[1][0], a[1][1], a[1][2]) + "]\n"
                "             [" + triple(a[2][0], a[2][1], a[2][2]) + "]>";
     });
+  mat33.attr("__matmul__") = mat33.attr("multiply");
 
   add_smat33<double>(m, "SMat33d");
   add_smat33<float>(m, "SMat33f");
@@ -171,6 +182,7 @@ void add_unitcell(py::module& m) {
     .def("approx", &Transform::approx);
 
   py::class_<Correlation>(m, "Correlation")
+    .def_readonly("n", &Correlation::n)
     .def("coefficient", &Correlation::coefficient)
     .def("mean_ratio", &Correlation::mean_ratio)
     ;
@@ -196,7 +208,10 @@ void add_unitcell(py::module& m) {
     });
   py::class_<Fractional, Vec3>(m, "Fractional")
     .def(py::init<double,double,double>())
+    .def("wrap_to_unit", &Fractional::wrap_to_unit)
     .def("__getitem__", (double (Fractional::*)(int) const) &Fractional::at)
+    .def(py::self + py::self)
+    .def(py::self - py::self)
     .def("__repr__", [](const Fractional& self) {
         return "<gemmi.Fractional(" + triple(self.x, self.y, self.z) + ")>";
     });
@@ -240,6 +255,12 @@ void add_unitcell(py::module& m) {
     .def_property_readonly("orthogonalization_matrix",
                            [](const UnitCell& self) { return self.orth.mat; })
     .def("set", &UnitCell::set)
+    .def("changed_basis_forward", &UnitCell::changed_basis_forward,
+         py::arg("op"), py::arg("set_images"))
+    .def("changed_basis_backward", &UnitCell::changed_basis_backward,
+         py::arg("op"), py::arg("set_images"))
+    .def("is_compatible_with_spacegroup", &UnitCell::is_compatible_with_spacegroup,
+         py::arg("sg"), py::arg("eps")=1e-3)
     .def("is_crystal", &UnitCell::is_crystal)
     .def("calculate_u_eq", &UnitCell::calculate_u_eq)
     .def("fractionalize", &UnitCell::fractionalize)
@@ -271,6 +292,7 @@ void add_unitcell(py::module& m) {
     .def("reciprocal_metric_tensor", &UnitCell::reciprocal_metric_tensor)
     .def("reciprocal", &UnitCell::reciprocal)
     .def("get_hkl_limits", &UnitCell::get_hkl_limits, py::arg("dmin"))
+    .def("g6", &UnitCell::g6, py::arg("centring_type"))
     .def(py::self == py::self)
     .def("__repr__", [](const UnitCell& self) {
         return "<gemmi.UnitCell(" + triple(self.a, self.b, self.c)
@@ -283,4 +305,34 @@ void add_unitcell(py::module& m) {
                              p[3].cast<double>(), p[4].cast<double>(), p[5].cast<double>());
          }
     ));
+
+  py::class_<GruberVector>(m, "GruberVector")
+    .def(py::init<const std::array<double,6>&>())
+    .def(py::init([](const UnitCell& u, const SpaceGroup* sg) {
+        return new GruberVector(u.g6(sg ? sg->centring_type() : 'P'));
+    }))
+    .def("parameters", &GruberVector::parameters)
+    .def_property_readonly("parameters", [](const GruberVector& g) {
+      return make_six_tuple(g.parameters());
+    })
+    .def("cell_parameters", [](const GruberVector& self) {
+      return make_six_tuple(self.cell_parameters());
+    })
+    .def("get_cell",
+         [](const GruberVector& self) { return new UnitCell(self.cell_parameters()); })
+    .def("is_normalized", &GruberVector::is_normalized)
+    .def("is_buerger", &GruberVector::is_buerger, py::arg("epsilon")=1e-9)
+    .def("normalize", &GruberVector::normalize, py::arg("epsilon")=1e-9)
+    .def("buerger_reduce", &GruberVector::buerger_reduce)
+    .def("niggli_step", &GruberVector::niggli_step, py::arg("epsilon"))
+    .def("niggli_reduce", &GruberVector::niggli_reduce,
+         py::arg("epsilon")=1e-9, py::arg("iteration_limit")=100)
+    .def("is_niggli", &GruberVector::is_niggli, py::arg("epsilon")=1e-9)
+    .def("__repr__", [](const GruberVector& self) {
+        using namespace std;  // VS2015/17 doesn't like std::snprintf
+        char buf[256];
+        snprintf(buf, 256, "<gemmi.GruberVector((%.2f, %.2f, %.2f, %.2f, %.2f, %.2f))>",
+                 self.A, self.B, self.C, self.xi, self.eta, self.zeta);
+        return std::string(buf);
+    });
 }
